<?php
/*
Plugin Name: Shopp Arrange
Plugin URI: http://clifgriffin.com
Description:  Allows categories, shipping, and payment options to be reordered seemlessly. 
Version: 1.0
Author: Clifton H. Griffin II
Author URI: http://clifgriffin.com
*/

class ShoppArrange {
	public function __construct()  
    {  
    	register_activation_hook( __FILE__, array($this, 'activate') );
    	
    	add_action('admin_menu', array($this,'admin_menu'), 11);
    	add_action('admin_print_scripts', array($this, 'print_scripts'));
    	add_action('wp_ajax_update_arrangement', array($this, 'update_arrangement'));
    	add_filter('wp_list_categories', array($this, 'add_category_id'));
    	add_action('admin_init', array($this, 'save_settings') );
    	
    	if(get_option('shopp_arrange_cat_override') == "yes")
	    	add_filter('get_terms_orderby', array($this, 'applyorderfilter'), 10, 2);
    	
    	add_filter('get_terms_args', array($this, 'addtaxonomytoargs'), 1, 2);
    	
    	if(get_option('shopp_arrange_shipping_override') == "yes")
	    	add_action('shopp_cart_retotal', array($this, 'change_shipping_options_order'), 100);
	    
	    if($_GET['page'] == "shopp-settings-preferences") {
	    	add_filter('gettext', array($this, 'add_settings'), 100, 3);
	    }
    }  
    
    function activate() {
		global $wpdb;
		
		$wpdb->show_errors();
	
		$query1 = $wpdb->query("SHOW COLUMNS FROM $wpdb->terms LIKE 'term_order'");
		
		if ($query1 == 0) {
			$wpdb->query("ALTER TABLE $wpdb->terms ADD `term_order` INT( 4 ) NULL DEFAULT '0'");
		}
		
		add_option('shopp_arrange_cat_override', 'yes');
	}
	
	function admin_menu() {
		add_submenu_page( null, "Category Order", "Category Order", "shopp_settings_update", "shopp-arrange-categories", array($this, 'admin_category_page') );
	}
	
	function admin_category_page()
	{
		include_once('shopp-arrange-categories-admin.php');
	}
	
	function add_category_id($content) {
		if ( isset($_GET['page']) && $_GET['page'] == "shopp-arrange-categories" ) {
			$content = preg_replace('/ cat-item-([0-9]*)/', '" id="$1', $content);
		}
		return $content;
	}
	
	function save_settings() {
		if(isset($_POST['new_order']) && isset($_GET['page']) && $_GET['page'] == "shopp-arrange-categories") {
			global $wpdb;
			$new_order = explode(",", $_POST['new_order']);
			
			foreach($new_order as $position => $term_id) {
				$wpdb->query($wpdb->prepare("UPDATE $wpdb->terms SET term_order = %d WHERE term_id = %d ", $position, $term_id));
			}
		}
		
		if(isset($_POST['shopp_arrange_shipping_override']) && $_GET['page'] == "shopp-settings-preferences") {
			update_option('shopp_arrange_shipping_override', $_REQUEST['shopp_arrange_shipping_override']);
		}
		
		if(isset($_POST['shopp_arrange_cat_override']) && $_GET['page'] == "shopp-settings-preferences") {
			update_option('shopp_arrange_cat_override', $_REQUEST['shopp_arrange_cat_override']);
		}
	}
	
	function applyorderfilter($orderby, $args)
	{
		if(isset($args['taxonomy']) && $args['taxonomy'] == 'shopp_category') {
			return 't.term_order';
		}
		else {
			return $orderby;
		}
	}
	
	function addtaxonomytoargs($args, $taxonomies) {
		if(in_array("shopp_category", $taxonomies)) {
			$args['taxonomy'] = $taxonomies[0];
		}
		
		return $args;
	}
    
    function print_scripts() {
	    if ( isset($_GET['page']) && ($_GET['page'] == "shopp-settings-payments" ||  $_GET['page'] == "shopp-settings-shipping" || $_GET['page'] == "shopp-arrange-categories") )
		{	
			wp_enqueue_script('jquery');
			wp_enqueue_script('jquery-ui-core');
			wp_enqueue_script('jquery-ui-sortable');
		}
		
		if(isset($_GET['page']) && ($_GET['page'] == "shopp-settings-payments" ||  $_GET['page'] == "shopp-settings-shipping")) {
			wp_enqueue_script( 'shopp_arrange', '/' . PLUGINDIR . '/shopp-arrange/js/arrange.js');
		}
		
		if(isset($_GET['page']) && ($_GET['page'] == "shopp-categories")) {
			wp_enqueue_script( 'shopp_arrange', '/' . PLUGINDIR . '/shopp-arrange/js/add_arrange_cat.js');
		}
    }
    
    function update_arrangement() {
    	global $wpdb;
    	$new_order = $_REQUEST['new_order'];
    	$new_order = explode(",", $new_order);
    	
    	if($_REQUEST['arrange'] == "shipping") {
	    	// Shipping options
	    	
	    	$new_setting = array();
	    	
	    	$new_setting[''] = true;
	    	
	    	foreach($new_order as $op) {
	    		$matched = preg_match('/[0-9]/', $op, $matches);
	    		
	    		$stripped_op = substr($op, 0, strpos($op, "-"));
	    		
		    	if($matched == 1) {
			    	$index = $matches[0];
			    	
			    	$new_setting[$stripped_op][$index] = 'true';
		    	} else {
			    	// Add normal
			    	$new_setting[$op] = 'true';
		    	}
	    	}
	    	
			$update_shipping = shopp_set_setting('active_shipping', serialize($new_setting));
			
			echo $update_shipping;
	    	
    	} else {
	    	// Payment options	
	    	$new_setting = array();
	    	$new_setting[] = '';
	    	
	    	foreach($new_order as $op) {
	    		$new_setting[] = $op;
	    	}
	    	
	    	$new_setting = join(",", $new_setting); 
	    	
	    	$update_payment = shopp_set_setting('active_gateways', $new_setting);
			
			echo $update_payment;
    	}
    	die();
    }

	function change_shipping_options_order() {
		$options = ShoppOrder()->Cart->shipping;
		
		uksort($options, array($this,'sort_override'));
		
		ShoppOrder()->Cart->shipping = $options;
	}
	
	function sort_override($a, $b) {	
		$active_shipping = shopp_setting('active_shipping');
		
		//get rid of the blank one
		array_shift($active_shipping);
		
		$a = substr($a, 0, strpos($a, "-"));
		$b = substr($b, 0, strpos($b, "-"));
		
		if($a == $b) return 0;
		
		foreach($active_shipping as $key => $op ) {
			if(stripos($key, $a) !== false) {
				return -1;
			} else if(stripos($key, $b) !== false) {
				return 1;
			}
		}
		
		return 0;
	}
	
	function add_settings($translated, $untranslated, $domain) {
		if($untranslated == "Product Downloads") {
			ob_start();
			require(dirname(__FILE__).'/settings-template.php');
			
			return ob_get_clean() . $translated;
		} else return $translated;
	}
}

$ShoppArrange = new ShoppArrange();